#!/usr/bin/env python
import os, time
# String functions
def keysort(key, revpoint = None):
    """
    Lambda function for sorting.
    This returns either the key as-is, or based on a point to reverse the ordering (e.g. IMGT 112),
    it provides the negative of the ord of the insertion.
    """
    if revpoint and key[0] == revpoint:
        return (key[0], -ord(key[1]))
    else:
        return key
def print_result(assignresults):
	lineout = []
	lineout.append('\t'.join(['Input','CDR','Sequence','Canonical','Median']))
	
	for i in range(len(assignresults)):
		if 'outputs' not in assignresults[i]: continue
		for cdr in sorted(assignresults[i]['outputs']):
			lineout.append('\t'.join([assignresults[i]['seqname']] + assignresults[i]['outputs'][cdr]))
	print(('\n'.join(lineout)))
# From BioPython
def SimpleFastaParser(handle):
	"""Generator function to iterate over Fasta records (as string tuples).
	
	For each record a tuple of two strings is returned, the FASTA title
	line (without the leading '>' character), and the sequence (with any
	whitespace removed). The title line is not divided up into an
	identifier (the first word) and comment or description.
	
	>>> with open("Fasta/dups.fasta") as handle:
		...     for values in SimpleFastaParser(handle):
		...         print(values)
		...
	('alpha', 'ACGTA')
	('beta', 'CGTC')
	('gamma', 'CCGCC')
	('alpha (again - this is a duplicate entry to test the indexing code)', 'ACGTA')
	('delta', 'CGCGC')
	
	"""
	# Skip any text before the first record (e.g. blank lines, comments)
	while True:
		line = handle.readline()
		if line == "":
			return  # Premature end of file, or just empty?
		if line[0] == ">":
			break
	while True:
		if line[0] != ">":
			raise ValueError(
				"Records in Fasta files should start with '>' character")
		title = line[1:].rstrip()
		lines = []
		line = handle.readline()
		while True:
			if not line:
				break
			if line[0] == ">":
				break
			lines.append(line.rstrip())
			line = handle.readline()
			
		# Remove trailing whitespace, and any internal spaces
		# (and any embedded \r which are possible in mangled files
		# when not opened in universal read lines mode)
		yield title, "".join(lines).replace(" ", "").replace("\r", "")
		if not line:
			return  # StopIteration
	assert False, "Should not reach this line"
# Write results
def write_txt(assignresults,outputdir, opf=""):
	lineout = ['\t'.join(['Input','CDR','Sequence','Canonical','Median'])]
	if opf == "": opf = os.path.join(outputdir,'{0}.txt'.format(time.time()))
	for i in range(len(assignresults)):
		for cdr in sorted(assignresults[i]['outputs']):
			lineout.append('\t'.join([assignresults[i]['seqname']] + assignresults[i]['outputs'][cdr]))
	if not os.path.exists(outputdir): os.mkdir(outputdir) 
	with open(opf,'wb') as f:
		f.writelines('\n'.join(lineout))
	return opf
	